#include "Background.h"

Background::Background()
{
	// complex
	compTempImage =	cvCreateImage(cvSize(CAM_W, CAM_H), 8, 3);
	compTempImageBW = cvCreateImage(cvSize(CAM_W, CAM_H), 8, 1);

	low_threshold_mask = new BwImage(cvCreateImage(cvSize(CAM_W, CAM_H), IPL_DEPTH_8U, 1));
	low_threshold_mask->Ptr()->origin = IPL_ORIGIN_BL;

	high_threshold_mask = new BwImage(cvCreateImage(cvSize(CAM_W, CAM_H), IPL_DEPTH_8U, 1));
	high_threshold_mask->Ptr()->origin = IPL_ORIGIN_BL;

	frame_count = 0;

	// simple
	bgImage = cvCreateImage(cvSize(CAM_W, CAM_H), 8, 3);
	tmpImage = cvCreateImage(cvSize(CAM_W, CAM_H), 8, 3);

	// general
	bgSet = false;
	subtractedImage = cvCreateImage(cvSize(CAM_W, CAM_H), 8, (CAM_COLOR == GL_RGB) ? 3 : 1);
}

Background::~Background(void)
{
	if (params)			delete params;
	if (bgs)			delete bgs;
	if (compBgImage)	delete compBgImage;

	cvReleaseImage(&subtractedImage);
	cvReleaseImage(&bgImage);
	cvReleaseImage(&tmpImage);
	cvReleaseImage(&compTempImage);
}

// ************************
// ***** COMPLEX **********
// ************************
void Background::initBackgroundComplex()
{
	params = new Algorithms::BackgroundSubtraction::AdaptiveMedianParams();
	params->SetFrameSize(CAM_W, CAM_H);
	params->LowThreshold() = 40;
	params->HighThreshold() = 2*params->LowThreshold();	// Note: high threshold is used by post-processing 
	params->SamplingRate() = 7;
	params->LearningFrames() = 30;

	bgs = new Algorithms::BackgroundSubtraction::AdaptiveMedianBGS();
	bgs->Initalize(*params);
}
void Background::setBackgroundImageComplex(IplImage* img)
{
	if (!bgSet)
	{
		if (img->nChannels == 1)	cvCvtColor(img, compTempImage, CV_GRAY2RGB);
		else						cvCopy(img, compTempImage);
		
		compBgImage = new RgbImage(compTempImage);
		compBgImage->ReleaseMemory(false);
		bgs->InitModel(*compBgImage); 

		printf("Background: is set\n");
		bgSet = true;
	}
	else
	{
		delete compBgImage;

		printf("Background: disabled\n");
		bgSet = false;
	}
}
void Background::subtractComplex(IplImage* img)
{
	if (bgSet)
	{
		RgbImage rgbImg(img);
		rgbImg.ReleaseMemory(false);

		bgs->Subtract(frame_count, rgbImg, *low_threshold_mask, *high_threshold_mask);
		bgs->Update(frame_count, rgbImg, *low_threshold_mask);

		cvAdd(low_threshold_mask->Ptr(), high_threshold_mask->Ptr(), compTempImageBW);

		BwImage bwImg(compTempImageBW);
		bwImg.ReleaseMemory(false);

		for (int j = 0; j < bwImg.Ptr()->height; j++)
			for (int k = 0; k < bwImg.Ptr()->width; k++)
			{
				// delete all pixels with the appr. same brightness
				if (bwImg(j, k) < 10)
					rgbImg(j, k, 0) = rgbImg(j, k, 1) = rgbImg(j, k, 2) = 0;
			}

		frame_count++;
	}
}

// ************************
// ****** SIMPLE **********
// ************************
void Background::setBackgroundImageSimple(IplImage* img)
{
	if (!bgSet)
	{
		cvCopy(img, bgImage);
		bgSet = true;
		printf("Background: is set\n");
	}
	else
	{
		cvReleaseImage(&bgImage);
		bgImage = cvCreateImage(cvSize(CAM_W, CAM_H), 8, 3);
		bgSet = false;
		printf("Background: disabled\n");
	}
}

void Background::subtractSimple(IplImage* img)
{
	if (bgSet)
	{

		RgbImage bg(bgImage);
		RgbImage fg(img);
		bg.ReleaseMemory(false);
		fg.ReleaseMemory(false);

		for (int i = 0; i < img->height; i++)
			for (int j = 0; j < img->width; j++)
			{
				// delete all pixels with the appr. same brightness
				if (abs(fg(i, j, 0) - bg(i, j, 0)) < 50)
					fg(i, j, 0) = fg(i, j, 1) = fg(i, j, 2) = 0;
			}
	}

}