#include "Gesture.h"

Gesture::Gesture(int radius, int xoffset, int yoffset)
{
	this->state = HAND_NONE;
	this->fPoints = NULL;
	this->pointTrans = 20;
	this->spreadTrans = 35;

	this->rad = radius;
	this->xOff = xoffset;
	this->yOff = yoffset;
	this->prevDepth = 0;

	fPoints = new std::vector<CvPoint>();

	tl = tr = bl = br = 0;

	// contour finding
	storage = cvCreateMemStorage(0);
	gestureImage = cvCreateImage(cvSize(CAM_W, CAM_H), 8, 3);
	tempImg = cvCreateImage(cvSize(CAM_W, CAM_H), 8, 1);
	tempImg2 = cvCreateImage(cvSize(CAM_W, CAM_H), 8, 1);

}

Gesture::~Gesture(void)
{
}


void Gesture::calcContours(IplImage* img)
{
	if (img->nChannels > 1)	cvCvtColor(img, tempImg, CV_RGB2GRAY);
	else					cvCopy(img, tempImg);

	cvZero(gestureImage);

	highpass(tempImg, tempImg2, 18, 4);
	amplify(tempImg2, tempImg, 16);

	cvFindContours(tempImg, storage, &contours, sizeof(CvContour), CV_RETR_TREE, CV_CHAIN_APPROX_SIMPLE, cvPoint(0,0));
	contours = cvApproxPoly(contours, sizeof(CvContour), storage, CV_POLY_APPROX_DP, 3, 1); 

	cvDrawContours(gestureImage, contours, CV_RGB(255, 0, 0), CV_RGB(0, 255, 0), 1, 1, CV_AA, cvPoint(0,0));

}
void Gesture::calcGesture(CvPoint center, float depth, int overlayLevel)
{
	cvFloodFill(gestureImage, center, CV_RGB(255, 255, 0), CV_RGB(255, 0, 0), CV_RGB(0, 0, 255));
}


void Gesture::highpass(IplImage* img, IplImage* tmp, float blur1, float blur2)
{
	if(blur1 > 0)
	cvSmooth(img, tmp, CV_BLUR , (int)(blur1 * 2) + 1);

	cvSub(img, tmp, tmp);

	if(blur2 > 0)
	cvSmooth(tmp, img, CV_BLUR , (int)(blur2 * 2) + 1);
}

void Gesture::amplify(IplImage* img, IplImage* destImg, float level)
{
	float scalef = level / 128.0f;

	cvMul(img, img, destImg, scalef);
}


void Gesture::calcCross(IplImage* stereoImg, CvPoint p, float depth, int overlayLevel)
{
	tl = tr = bl = br = 0;

	// top left
	resetCxCy(p);
	//while(isHigher(stereoImg, c_x, c_y, depth)) { tl++;  c_x = c_y -= 1; }

	// top right
	resetCxCy(p);
	//while(isHigher(stereoImg, c_x, c_y, depth)) { tr++;  c_x += 1; c_y -= 1; }

	// bottom left
	resetCxCy(p);
	//while(isHigher(stereoImg, c_x, c_y, depth)) { bl++;  c_x -= 1; c_y += 1; }

	// bottom right
	resetCxCy(p);
	//while(isHigher(stereoImg, c_x, c_y, depth)) { br++;  c_x = c_y += 1; }

	printf("%i, %i, %i, %i\n", tl, tr, bl, br);

	// draw
	cvLine(stereoImg, cvPoint(c_x, c_y), cvPoint(c_x - 50, c_y - 50),	CV_RGB(255, 255, 255));
	cvLine(stereoImg, cvPoint(c_x, c_y), cvPoint(c_x - 50, c_y + 50),	CV_RGB(255, 255, 255));
	cvLine(stereoImg, cvPoint(c_x, c_y), cvPoint(c_x + 50, c_y - 50),	CV_RGB(255, 255, 255));
	cvLine(stereoImg, cvPoint(c_x, c_y), cvPoint(c_x + 50, c_y + 50),	CV_RGB(255, 255, 255));
}

void Gesture::resetCxCy(CvPoint p)
{
	c_x = p.x / DISP_CAM_FACTOR;
	c_y = p.y / DISP_CAM_FACTOR;
}

bool Gesture::isHigher(IplImage* img, int x, int y, float depth)
{
	return (	c_x > 0 
			&&	c_y > 0
			&&	c_x < img->width  
			&&	c_y < img->height 
			&& (float)(cvGet2D(img, c_x, c_y).val[0] / 255.f) > depth - .1f);
}

void Gesture::initRadial(int radius, int xoffset, int yoffset)
{
	fPoints->clear();

	// etwas mehr als die obere kreishlfte berechnen
	for (double val = (PI/8); val < PI; val+=0.05)
		fPoints->push_back(cvPoint(	(int)((cos(val) * radius) + xoffset),
									(int)((sin(val) * radius) - yoffset)));
}
void Gesture::calcRadial(IplImage* stereoImg, CvPoint p, float depth, int overlayLevel)
{
	if (abs(prevDepth - depth) > .1)
	{
		initRadial((int)((float)rad * ((depth / 3) + .5f)), xOff, yOff);
		prevDepth = depth;
	}

	actBright = 0;
	white_count = 0;
	
	// etwas mehr als die obere kreishlfte berechnen
	for (int i = 0; i < fPoints->size(); i++)
	{
		c_x = (int)(p.x / DISP_CAM_FACTOR) + fPoints->at(i).x;
		c_y = (int)(p.y / DISP_CAM_FACTOR) - fPoints->at(i).y;
		
		if (c_x < stereoImg->width && c_y < stereoImg->height && c_x > 0 && c_y > 0)
		{
			// pixelhelligkeit holen
			actBright = float(cvGet2D(stereoImg, c_y, c_x).val[0]) / 255.f; //std::cout << actBright << std::endl;
			
			if (actBright > depth - 0.1)	
				white_count++;

			// zeichnen
			if (overlayLevel > 0)
				cvSet2D(stereoImg, c_y,  c_x , CV_RGB(255, 255, 255));
		}
	}

	if (white_count > spreadTrans)		state = HAND_SPREAD;
	else if (white_count < pointTrans)	state = HAND_POINT;
	else								state = HAND_NORMAL;

	printf("%s : %i\n", state, white_count);
}