#include "HVCamera.h"

HVCamera::HVCamera(videoInput *vi, int device, float* distTopOffset, float* distBottomOffset)
{
	this->vi = vi;
	this->device = device;
	this->showCenter = false;

	// set up distortion values
	this->distTopOffset = distTopOffset;
	this->distBottomOffset = distBottomOffset;

	src_quad[0].x = 0; src_quad[0].y = 0; //top-left
	src_quad[1].x = (float)CAM_W; src_quad[1].y = 0; //top-right
	src_quad[2].x = (float)CAM_W; src_quad[2].y = (float)CAM_H; //bottom-right
	src_quad[3].x = 0; src_quad[3].y = (float)CAM_H; //bottom-left

	dest_quad[0].x = -*distTopOffset; dest_quad[0].y = 0;
	dest_quad[1].x = (float)CAM_W+(*distTopOffset); dest_quad[1].y = 0;
	dest_quad[2].x = (float)CAM_W+(*distBottomOffset); dest_quad[2].y = (float)CAM_H;
	dest_quad[3].x = -*distBottomOffset; dest_quad[3].y = (float)CAM_H;

	// vi device setup
	vi->setupDevice(device, CAM_W, CAM_H);
	//vi->setIdealFramerate(device, 30);
	
	// image im speicher erstellen
	rawImage = cvCreateImage(cvSize(CAM_W, CAM_H), 8, 3);
	grayImage = cvCreateImage(cvSize(CAM_W, CAM_H), 8, 1);
	image = cvCreateImage(cvSize(CAM_W, CAM_H), 8, (CAM_COLOR == GL_RGB) ? 3 : 1);

	// create background subtraction
	bg = new Background();
	bg->initBackgroundComplex();

	bgComplexMode = false;
}

HVCamera::~HVCamera(void)
{
	vi->stopDevice(device);
	delete bg;

	cvReleaseImage(&rawImage);
	cvReleaseImage(&grayImage);
	cvReleaseImage(&image);
}

void HVCamera::run(void)
{
	vi->getPixels(device, (unsigned char*)rawImage->imageData, true, true);
	
	if (bgComplexMode)	bg->subtractComplex(rawImage);
	else				bg->subtractSimple(rawImage);

	if (image->nChannels > 1)	cvCopy(rawImage, image);
	else						cvCvtColor(rawImage, image, CV_BGR2GRAY);



	//cvEqualizeHist(grayImage, image);

	// distortion
	/*
	dest_quad[0].x = -*distTopOffset;
	dest_quad[1].x = (float)CAM_W+(*distTopOffset);
	dest_quad[2].x = (float)CAM_W+(*distBottomOffset);
	dest_quad[3].x = -*distBottomOffset;

	CvMat* matrix = cvCreateMat(3, 3, CV_32FC1);
	cvWarpPerspectiveQMatrix(&src_quad[0], &dest_quad[0], matrix);
	
	cvWarpPerspective(image, image, matrix);
	*/
}

void HVCamera::draw(IplImage* img)
{
	// draw center
	if (showCenter)
	{
		cvDrawLine(image, cvPoint((CAM_W/2)-10, (CAM_H/2)), cvPoint((CAM_W/2)+10, (CAM_H/2)), CV_RGB(255, 255, 255));
		cvDrawLine(image, cvPoint((CAM_W/2), (CAM_H/2)-30), cvPoint((CAM_W/2), (CAM_H/2)+30), CV_RGB(255, 255, 255));
	}
}


void HVCamera::setBg(bool complex)
{
	if (complex)
	{
		bgComplexMode = true;
		bg->setBackgroundImageComplex(rawImage);
	}
	else
	{
		bgComplexMode = false;
		bg->setBackgroundImageSimple(rawImage);
	}
}