#include "Common.h"
#include "CubicleWrapper.h"
#include "Skincolor.h"
#include "LearnedColor.h"
#include "OpticalFlow.h"
#include "CamShift.h"
#include "Undistortion.h"
#include "VisionConductorTwin.h"
#include "GestureServer.h"
#include "HandVULite.h"

#include <iostream>

#include <string.h>
#include <sstream>

// Time-Subclass ---------------------------------------------------------------- START
RefTime RefClockWin::GetCurrentTimeUsec() const
{
  clock_t ct = clock();
  return (RefTime) (ct/CLOCKS_PER_USEC);
}
// Time-Subclass ---------------------------------------------------------------- END


HandVuLite::HandVuLite():HandVu()
{
	m_pConductor = new VisionConductorTwin();
	state = HAND_NONE;
	procImage = cvCreateImage(cvSize(CAM_W, CAM_H), 8, 3);
	secondImg = cvCreateImage(cvSize(CAM_W, CAM_H), 8, 3);
	tempImg = cvCreateImage(cvSize(CAM_W, CAM_H), 8, 3);

	maxRadius = 105;
	xoffset = 0;
	yoffset = 0;
	transValue = .9f;
	amplifyEnabled = true;
	amplifyValue = 25;
	fingerDetectionEnabled = true;

	fPoints = new std::vector<CvPoint>();
}

void HandVuLite::initLite(char* conductorPath, int overlayLevel)
{
	hvClock = new RefClockWin();
	Initialize(CAM_W, CAM_H, hvClock, NULL);
	LoadConductor(conductorPath);
	SetOverlayLevel(overlayLevel);
	initFingers();
}

HandVuLite::~HandVuLite(void)
{
	cvReleaseImage(&procImage);
	HandVu::~HandVu();
}

// own functions
float HandVuLite::getCenterX(void){ return m_center_pos.x; }
float HandVuLite::getCenterY(void){ return m_center_pos.y; }
double HandVuLite::getCenterDepth(void){ return m_center_depth; }
char* HandVuLite::getFingerState(void){ return state; }
bool HandVuLite::isTracking(void){ return m_tracking; }


// #################### PROCESS ####################
void HandVuLite::processHv(IplImage* lImg, IplImage* rImg, float depth)
{
	if (lImg->nChannels == 1)	cvCvtColor(lImg, procImage, CV_GRAY2BGR);
	else						cvCopy(lImg, procImage);

	hvRefTime = hvClock->GetCurrentTimeUsec();
	GrabbedImage gi(procImage, hvRefTime, -1);

	if (rImg)
	{
		cvCvtColor(rImg, secondImg, CV_GRAY2BGR);
		ProcessFrame(gi, secondImg);
	}
	else
		ProcessFrame(gi);

	if (amplifyEnabled)
	{
		cvCopy(procImage, tempImg);
		highpass(procImage, tempImg, 18, 4);
		amplify(tempImg, procImage, amplifyValue);
	}

	if (m_tracking)	calcFingerState(procImage, depth);
	else			state = HAND_NONE;
}

// #################### CALC FINGERS ####################
void HandVuLite::initFingers()
{
	fPoints->clear();

	// etwas mehr als die obere kreishlfte berechnen
	for (float i = 0; i < PI+(PI/8); i+=0.05f)
		fPoints->push_back(cvPoint(	(int)((cos(i) * maxRadius) + xoffset),
									(int)((sin(i) * maxRadius) - yoffset)));
}

/** calculates the fingers two times **/
void HandVuLite::calcFingerState(IplImage* rgb_img, float depth)
{
	if (fingerDetectionEnabled)
	{
		int fingers = calcFingers(rgb_img, depth);
		int fingers2 = calcFingers(rgb_img, depth + (.3f * depth), 2, 5, 2);

		int final = max(fingers, fingers2);
		
		if (fingers2 == 1) state = HAND_POINT;
		else if (abs(fingers - fingers2) < 3)
		{
			if (final > 3)		state = HAND_SPREAD;
			else				state = HAND_NORMAL;
		}
		else
			state = HAND_NORMAL;
	}
	else 
		state = HAND_NORMAL;
}

int HandVuLite::calcFingers(IplImage* rgb_img, float depth, int minWhitePoints, int maxWhitePoints, int minBlackPoints)
{	
	int	fingers = 0;

	actBright = 0;
	white_count = 0;
	black_count = 0;
	
	// etwas mehr als die obere kreishlfte berechnen
	for (int i = 0; i < fPoints->size(); i++)
	{
		c_x = (int)(m_center_pos.x + (fPoints->at(i).x * depth));
		c_y = (int)(m_center_pos.y - (fPoints->at(i).y * depth));
		
		if (c_x < rgb_img->width && c_y < rgb_img->height && c_x > 0 && c_y > 0)
		{
			// pixelhelligkeit holen
			actBright = float(cvGet2D(rgb_img, c_y, c_x).val[0]) / 255.f;
			
			if (actBright > transValue)	
			{
				black_count = 0;
				white_count++;
			}
			else
			{
				black_count++;
			}
			
			if (white_count >= minWhitePoints && white_count <= maxWhitePoints && black_count >= minBlackPoints)
			{
				white_count = 0;
				black_count = 0;
				fingers++;
			}
			else if (black_count > 1)
			{
				black_count = 0;
				white_count = 0;
			}

			// zeichnen
			if (m_overlay_level > 0)
			{
				if (actBright > transValue)	cvSet2D(rgb_img, c_y,  c_x , CV_RGB(0, 0, 255));
				else						cvSet2D(rgb_img, c_y,  c_x , CV_RGB(0, 255, 0));
			}
		}
	}
	
	//fingers = (int)ceil((float)fingers / 2);



	return fingers;
}

void HandVuLite::amplify(IplImage* img, IplImage* destImg, float level)
{
	float scalef = level / 128.0f;

	cvMul(img, img, destImg, scalef);
}

void HandVuLite::highpass(IplImage* img, IplImage* tmp, float blur1, float blur2)
{
	if(blur1 > 0)
	cvSmooth(img, tmp, CV_BLUR, (int)(blur1 * 2) + 1);

	cvSub(img, tmp, tmp);

	if(blur2 > 0)
	cvSmooth(tmp, img, CV_BLUR, (int)(blur2 * 2) + 1);
}



/** hauptschlich aus der Hauptklasse kopiert, nur einige Befehle wurden weggelassen **/
HandVu::HVAction HandVuLite::ProcessFrame(GrabbedImage& inOutImage, const IplImage* rightImage)
{
	m_rgbImage = inOutImage.GetImage();
	m_sample_time = inOutImage.GetSampleTime();

	// sanity checks
	if(!m_initialized) {
		throw HVException("HandVu not initialized, cannot process");
	}
	if (m_rgbImage->width!=m_img_width || m_rgbImage->height!=m_img_height) {
		throw HVException("image dimensions do not match initialization");
	}
	VERBOSE2(5, "HandVu: processing frame (active: %s, tracking: %s)",
		m_active?"yes":"no", m_tracking?"yes":"no");

	if (m_rgbImage->origin==1) {
		m_rgbImage->origin = 0;
		cvMirror(m_rgbImage, NULL, 0);
	}

	// check latency of the incoming frame - the result is used
	// after KLT tracking
	HVAction action = CheckLatency();

	// return if we're not supposed to do anything
	if (!m_active
		|| m_scan_area.left>=m_scan_area.right 
		|| m_scan_area.top>=m_scan_area.bottom) 
	{
		if (action==HV_PROCESS_FRAME) 
		{
			// adjust exposure
			//CheckAndCorrectExposure();

			// undistort image, adjust location of centroid
			if (m_undistort) {
				m_pUndistortion->Undistort(m_rgbImage);
				    //m_pUndistortion->Transform(centroid);
			}
		}

		KeepStatistics(action);
		DrawOverlay();
		SendEvent();

		return action;
	}

	// set current buffer
	m_buf_indx_cycler = 1-m_buf_indx_cycler;
	m_curr_buf_indx = m_buf_indx_cycler;
	m_prev_buf_indx = 1-m_buf_indx_cycler;

	// create gray scale image
	{
		int scan_width = m_scan_area.right-m_scan_area.left;
		int cvt_left = max(0, m_scan_area.left-scan_width/2);
		int cvt_width = min(m_scan_area.right+scan_width/2, m_rgbImage->width)-cvt_left;
		int scan_height = m_scan_area.bottom-m_scan_area.top;
		int cvt_top = max(0, m_scan_area.top-scan_height/2);
		int cvt_height = min(m_scan_area.bottom+scan_height/2, m_rgbImage->height)-cvt_top;
		ASSERT(cvt_height>0 && cvt_width>0);
		if (!(cvt_height>0 && cvt_width>0)) {
			KeepStatistics(action);
			DrawOverlay();
			SendEvent();

			return action;
		}
		cvSetImageROI(m_rgbImage, cvRect(cvt_left, cvt_top, cvt_width, cvt_height));
		cvSetImageROI(m_grayImages[m_curr_buf_indx], cvRect(cvt_left, cvt_top, cvt_width, cvt_height));

		cvCvtColor(m_rgbImage, m_grayImages[m_curr_buf_indx], CV_BGR2GRAY);

		cvResetImageROI(m_rgbImage);
		cvResetImageROI(m_grayImages[m_curr_buf_indx]);
	}

	// do the all-important, fast KLT tracking
	m_recognized = false;
	if (m_tracking) {    
		m_tracking = DoTracking();
		if (!m_tracking) {
			// lost tracking
			StartRecognition();
		}
	}

	// take the recommendation from CheckLatency to heart
	if (action!=HV_PROCESS_FRAME) {
		KeepStatistics(action);
		DrawOverlay();
		SendEvent();

		return action;
	}

	if (m_tracking) {    
		m_recognized = DoRecognition();
		FindSecondHand();

	} else {
		m_recognized = DoDetection();
	}

	if (m_recognized && m_do_track) {
		InitializeTracking();
		m_tracking = true;
	}

	// restrict the general color segmentation ROI; this is
	// not really important for anything but high overlay_levels
	int scan_left = max(0, m_scan_area.left);
	int scan_right = min(m_scan_area.right, m_rgbImage->width);
	int scan_top = max(0, m_scan_area.top);
	int scan_bottom = min(m_scan_area.bottom, m_rgbImage->height);
	CRect scan_area(scan_left, scan_top, scan_right, scan_bottom);

	// use stereo correspondence to obtain distance of hand from camera
	if (m_tracking && rightImage) {
		int sz = (scan_right-scan_left)/5;
		CvRect area = cvRect((int)m_center_pos.x-sz, (int)m_center_pos.y-sz, 2*sz, 2*sz);
		CalculateDepth(rightImage, area);
	}

	// adjust exposure
	CheckAndCorrectExposure();

	// drawing

	if (m_tracking) {
		m_pLearnedColor->DrawOverlay(m_rgbImage, m_overlay_level, scan_area);
	}

	m_pSkincolor->DrawOverlay(m_rgbImage, m_overlay_level, CRect(m_last_match));

	m_pCubicle->DrawOverlay(m_rgbImage, m_overlay_level);

	if (m_tracking) {
		if (m_pConductor->m_tr_type & VisionConductor::VC_TT_CAMSHIFT) {
			m_pCamShift->DrawOverlay(m_rgbImage, m_overlay_level);
		} else {
			m_pOpticalFlow->DrawOverlay(m_rgbImage, m_overlay_level);
		}
	}
	// undistort image, adjust location of centroid
	if (m_undistort) {
		m_pUndistortion->Undistort(m_rgbImage);
		   //m_pUndistortion->Transform(centroid);
	}

	KeepStatistics(action);
	DrawOverlay();
	SendEvent();

	return action;
}