#pragma once

#include "stdafx.h"
#include <time.h>
#include "HandVu.hpp"
#include "VisionConductorTwin.h"

#define	PI 3.141592654

// Time-Subclass ---------------------------------------------------------------- START
#define CLOCKS_PER_USEC ((double)(CLOCKS_PER_SEC)/1000000.0)

class RefClockWin : public RefClock 
{
public:
  virtual RefTime GetCurrentTimeUsec() const;
};
// Time-Subclass ---------------------------------------------------------------- END

#define HAND_NONE "HAND_NONE"
#define HAND_NORMAL "HAND_NORMAL"
#define HAND_POINT "HAND_POINT"
#define HAND_SPREAD "HAND_SPREAD"


class HandVuLite : public HandVu
{
public:
	HandVuLite::HandVuLite();
	HandVuLite::~HandVuLite();
	HandVu::HVAction ProcessFrame(GrabbedImage& inOutImage, const IplImage* rightImage = NULL);

	// own functions
	float getCenterX(void);
	float getCenterY(void);
	double getCenterDepth(void);
	char* getFingerState(void);
	bool isTracking(void);

	void amplify(IplImage* img, IplImage* destImg, float level);
	void highpass(IplImage* img, IplImage* tmp, float blur1, float blur2);

	void initLite(char* conductorPath, int overlayLevel);
	void initFingers();
	int calcFingers(IplImage* rgb_img, float depth = 1, int minTransPoints = 3, int maxTransPoints = 10, int minBlackPoints = 2);
	void calcFingerState(IplImage* rgb_img, float depth = 1);
	void processHv(IplImage* lImg, IplImage* rImg = NULL, float depth = 1);
protected:
	// amplify & highpass temp image
	IplImage*	tempImg;

	// process vars
	RefClockWin*	hvClock;
	RefTime			hvRefTime;
	IplImage*		secondImg;

	// ### finger detection ###
	// circle
	std::vector<CvPoint>* fPoints;
	// process vars
	CvPoint			circle;
	int				c_x, c_y;
	float			actBright;
	int				white_count;
	int				black_count;
	// final state
	char*			state;

	float			prevDepth;
public:
	int				maxRadius, xoffset, yoffset;
	float			transValue;
	bool			amplifyEnabled;
	float			amplifyValue;
	bool			fingerDetectionEnabled;
	IplImage*		procImage;
	// Attribut mit der Zwillings-Klasse berschreiben, um HandVuLite zugriff auf die protected vars zu geben
	VisionConductorTwin*        m_pConductor;
};