#pragma once

#include "stdafx.h"
#include <vector>

class StereoView
{
public:
	StereoView(int numBoards = 10, int waitFramesForNextBoard = 100, int cornersX = 10, int cornersY = 7);
	~StereoView(void);

	// disparity
	void	calcDisparity(IplImage* lImg, IplImage* rImg);
	void	calcDepth(int centerX, int centerY, int size = 30);

	// chessboard calibration
	void	resetAnalyse(void);
	bool	handleCalibration(IplImage *lImg, IplImage *rImg);
	bool	analyseImages(IplImage* lImg, IplImage* rImg);
	void	createRectification(void);
	void	calibrateImages(IplImage* lImg, IplImage* rImg);
	void	highpassImage(IplImage* img);

	// save & load
	bool	loadCalibration(char* filename);
	bool	saveCalibration(char* filename);

	void	setDispValues(int pos, unsigned int range);
	unsigned int getDispRange(void);
	int		getDispPos(void);

	void	draw(int overlayLevel);

	// lines
	void	drawCalibrationLines(IplImage* img, int count = 20);
public:
	IplImage*				disparityImage;
	IplImage*				highpassedImage;
	float					depth;
protected:
	// disparity image
	IplImage				*leftTmpImg, *rightTmpImg;
	IplImage				*tmpImage, *tmpGrayImage;
	CvStereoBMState*		state;
	unsigned int			disparityRange; /* 1,2,3.. */
	int						disparityPosition; /*...-2,-1,0,1,2... */

	// highpass 
	IplImage*				hpTmpImg;

	// chessboard calibration
	IplImage*				lGrayTmpImage;
	IplImage*				rGrayTmpImage;
	// # general
	bool					ready;
	int						waitFrames;
	int						cornersX, cornersY, nCorners, numBoards;
	int						frame, successes;
	CvSize					boardSize, imageSize;
	// # points
	std::vector<CvPoint2D32f>	pointsTemp[2];
	std::vector<CvPoint3D32f>	objectPoints;
	std::vector<CvPoint2D32f>	points[2];
	std::vector<int>			npoints;
	// # matrices
	CvMat						*mx1,*my1,*mx2,*my2;

	// draw
	CvRect					depthRect;
};
