#include "WindowOutput.h"

WindowOutput::WindowOutput(char* title, int startX, int startY)
{
	// init params
	//this->additionalTitle = "";
	this->title = title;
	this->prevImages = 0;

	// glfw
	glfwInit();

	if (!glfwOpenWindow(CAM_W, CAM_H, 24, 24, 24, 24, 24, 24, GLFW_WINDOW))
		return;

	glfwSetWindowTitle("Initialising ...");
	glfwSetWindowPos(startX, startY);
	glfwSwapInterval(1);

	// imageTextures
	it[0] = new imageTexture(CAM_W, CAM_H, CAM_COLOR);
	it[1] = new imageTexture(CAM_W, CAM_H, CAM_COLOR);
	it[2] = new imageTexture(CAM_W, CAM_H, GL_LUMINANCE);
	it[3] = new imageTexture(CAM_W, CAM_H, GL_RGB);

	// fps counter
	frames = 0;
	startTime = glfwGetTime();
}

WindowOutput::~WindowOutput(void)
{
	glfwTerminate();
}



void WindowOutput::calcFps(void)
{
	time = glfwGetTime();
	if((time-startTime) > 1.0 || frames == 0 )
	{
		fps = (double)frames / (time-startTime);
		startTime = time;
		frames = 0;
	}

	// output to window
	sprintf_s(windowTitle, "hv11 (%.1f FPS) %s", fps, additionalTitle);
	glfwSetWindowTitle(windowTitle);

	frames ++;
}


void WindowOutput::process(IplImage* img)
{
	checkWinSize(1);

	glViewport(0, 0, CAM_WIN_W, CAM_WIN_H); 
	glClearColor(0.0f, 0.0f, 0.0f, 0.0f);
	glClear(GL_COLOR_BUFFER_BIT);
	setupScreen();
	
	it[3]->loadImageData((unsigned char*)img->imageData, CAM_W, CAM_H, GL_RGB);
	it[3]->renderTexture(0, 0, CAM_WIN_W, CAM_WIN_H);

    glfwSwapBuffers();
}
void WindowOutput::process(IplImage* lImg, IplImage* rImg)
{
	checkWinSize(2);

	glViewport(0, 0, CAM_WIN_W * 2, CAM_WIN_H);
	glClearColor(0.0f, 0.0f, 0.0f, 0.0f);
	glClear(GL_COLOR_BUFFER_BIT);
	setupScreen();
	
	it[0]->loadImageData((unsigned char*)lImg->imageData, CAM_W, CAM_H, CAM_COLOR);
	it[0]->renderTexture(0, 0, CAM_WIN_W, CAM_WIN_H);

	it[1]->loadImageData((unsigned char*)rImg->imageData, CAM_W, CAM_H, CAM_COLOR);
	it[1]->renderTexture(CAM_WIN_W, 0, CAM_WIN_W, CAM_WIN_H);

    glfwSwapBuffers();
}
void WindowOutput::process(IplImage* lImg, IplImage* rImg, IplImage* disparityImage)
{
	checkWinSize(3);

	glViewport(0, 0, CAM_WIN_W * 2, CAM_WIN_H * 2);
	glClearColor(0.0f, 0.0f, 0.0f, 0.0f);
	glClear(GL_COLOR_BUFFER_BIT);
	setupScreen();
	
	it[0]->loadImageData((unsigned char*)lImg->imageData, CAM_W, CAM_H, CAM_COLOR);
	it[0]->renderTexture(0, 0, CAM_WIN_W, CAM_WIN_H);

	it[1]->loadImageData((unsigned char*)rImg->imageData, CAM_W, CAM_H, CAM_COLOR);
	it[1]->renderTexture(CAM_WIN_W, 0, CAM_WIN_W, CAM_WIN_H);

	it[2]->loadImageData((unsigned char*)disparityImage->imageData, CAM_DISPARITY_W, CAM_DISPARITY_H, GL_LUMINANCE);
	it[2]->renderTexture(0, CAM_WIN_H, CAM_WIN_W, CAM_WIN_H);

    glfwSwapBuffers();
}
void WindowOutput::process(IplImage* lImg, IplImage* rImg, IplImage* disparityImage, IplImage* trackImage)
{
	checkWinSize(4);

	glViewport(0, 0, CAM_WIN_W * 2, CAM_WIN_H * 2);
	glClearColor(0.0f, 0.0f, 0.0f, 0.0f);
	glClear(GL_COLOR_BUFFER_BIT);
	setupScreen();
	
	it[0]->loadImageData((unsigned char*)lImg->imageData, CAM_W, CAM_H, CAM_COLOR);
	it[0]->renderTexture(0, 0, CAM_WIN_W, CAM_WIN_H);

	it[1]->loadImageData((unsigned char*)rImg->imageData, CAM_W, CAM_H, CAM_COLOR);
	it[1]->renderTexture(CAM_WIN_W, 0, CAM_WIN_W, CAM_WIN_H);

	it[2]->loadImageData((unsigned char*)disparityImage->imageData, CAM_DISPARITY_W, CAM_DISPARITY_H, GL_LUMINANCE);
	it[2]->renderTexture(0, CAM_WIN_H, CAM_WIN_W, CAM_WIN_H);

	it[3]->loadImageData((unsigned char*)trackImage->imageData, CAM_W, CAM_H, GL_RGB);
	it[3]->renderTexture(CAM_WIN_W, CAM_WIN_H, CAM_WIN_W, CAM_WIN_H);

    glfwSwapBuffers();
}

void WindowOutput::process(IplImage* lImg, IplImage* rImg, IplImage* disparityImage, IplImage* trackImage, IplImage* gestureImage)
{
	checkWinSize(5);

	glViewport(0, 0, CAM_WIN_W * 3, CAM_WIN_H * 2);
	glClearColor(0.0f, 0.0f, 0.0f, 0.0f);
	glClear(GL_COLOR_BUFFER_BIT);
	setupScreen();
	
	it[0]->loadImageData((unsigned char*)lImg->imageData, CAM_W, CAM_H, CAM_COLOR);
	it[0]->renderTexture(0, 0, CAM_WIN_W, CAM_WIN_H);

	it[1]->loadImageData((unsigned char*)rImg->imageData, CAM_W, CAM_H, CAM_COLOR);
	it[1]->renderTexture(CAM_WIN_W, 0, CAM_WIN_W, CAM_WIN_H);

	it[2]->loadImageData((unsigned char*)disparityImage->imageData, CAM_DISPARITY_W, CAM_DISPARITY_H, GL_LUMINANCE);
	it[2]->renderTexture(0, CAM_WIN_H, CAM_WIN_W, CAM_WIN_H);

	it[3]->loadImageData((unsigned char*)trackImage->imageData, CAM_W, CAM_H, GL_RGB);
	it[3]->renderTexture(CAM_WIN_W, CAM_WIN_H, CAM_WIN_W, CAM_WIN_H);

	it[3]->loadImageData((unsigned char*)gestureImage->imageData, CAM_W, CAM_H, GL_RGB);
	it[3]->renderTexture(CAM_WIN_W * 2, CAM_WIN_H, CAM_WIN_W, CAM_WIN_H);

    glfwSwapBuffers();
}

void WindowOutput::checkWinSize(int actImages)
{
	if (actImages != prevImages)
	{
		switch(actImages)
		{
			case 1: glfwSetWindowSize(CAM_WIN_W,	CAM_WIN_H); break;
			case 2: glfwSetWindowSize(CAM_WIN_W * 2,CAM_WIN_H); break;
			case 3: glfwSetWindowSize(CAM_WIN_W * 2,CAM_WIN_H * 2); break;
			case 4: glfwSetWindowSize(CAM_WIN_W * 2,CAM_WIN_H * 2); break;
			case 5: glfwSetWindowSize(CAM_WIN_W * 3,CAM_WIN_H * 2); break;
		}
		prevImages = actImages;
	}
}

void WindowOutput::setupScreen(void)
{
	int w, h;
	
	glfwGetWindowSize(&w, &h);
	
	float halfFov, theTan, screenFov, aspect;

	screenFov = 60.0f;
	
	float eyeX = (float)w / 2.0f;
	float eyeY = (float)h / 2.0f;
	halfFov = (float)(PI * screenFov) / 360.0f;
	theTan = tanf(halfFov);
	float dist = eyeY / theTan;
	float nearDist = dist / 10.0f;
	float farDist = dist * 10.0f;
	aspect = (float)w/(float)h;

	glMatrixMode(GL_PROJECTION);
	glLoadIdentity();
	gluPerspective(-screenFov, aspect, nearDist, farDist);
	gluLookAt(eyeX, eyeY, -dist, eyeX, eyeY, 0.0, 0.0, 1.0, 0.0);
	glMatrixMode(GL_MODELVIEW);
	glLoadIdentity();
}